@@ -7,6 +7,7 @@
 #include "dolphincontextmenu.h"
 
 #include "dolphin_generalsettings.h"
+#include "dolphin_contextmenusettings.h"
 #include "dolphinmainwindow.h"
 #include "dolphinnewfilemenu.h"
 #include "dolphinplacesmodelsingleton.h"
@@ -61,6 +62,8 @@ DolphinContextMenu::DolphinContextMenu(DolphinMainWindow* parent,
     // or the items itself. To increase the performance both lists are cached.
     const DolphinView* view = m_mainWindow->activeViewContainer()->view();
     m_selectedItems = view->selectedItems();
+
+    installEventFilter(this);
 }
 
 DolphinContextMenu::~DolphinContextMenu()
@@ -109,20 +112,28 @@ DolphinContextMenu::Command DolphinContextMenu::open()
     return m_command;
 }
 
-void DolphinContextMenu::keyPressEvent(QKeyEvent *ev)
+void DolphinContextMenu::childEvent(QChildEvent* event)
 {
-    if (m_removeAction && ev->key() == Qt::Key_Shift) {
-        m_removeAction->update(DolphinRemoveAction::ShiftState::Pressed);
+    if(event->added()) {
+        event->child()->installEventFilter(this);
     }
-    QMenu::keyPressEvent(ev);
+    QMenu::childEvent(event);
 }
 
-void DolphinContextMenu::keyReleaseEvent(QKeyEvent *ev)
+bool DolphinContextMenu::eventFilter(QObject* dest, QEvent* event)
 {
-    if (m_removeAction && ev->key() == Qt::Key_Shift) {
-        m_removeAction->update(DolphinRemoveAction::ShiftState::Released);
+    if(event->type() == QEvent::KeyPress || event->type() == QEvent::KeyRelease) {
+        QKeyEvent* keyEvent = static_cast<QKeyEvent*>(event);
+        if(m_removeAction && keyEvent->key() == Qt::Key_Shift) {
+            if(event->type() == QEvent::KeyPress) {
+                m_removeAction->update(DolphinRemoveAction::ShiftState::Pressed);
+            } else {
+                m_removeAction->update(DolphinRemoveAction::ShiftState::Released);
+            }
+            return true;
+        }
     }
-    QMenu::keyReleaseEvent(ev);
+    return QMenu::eventFilter(dest, event);
 }
 
 void DolphinContextMenu::openTrashContextMenu()
@@ -175,31 +186,33 @@ void DolphinContextMenu::openTrashItemContextMenu()
 void DolphinContextMenu::addDirectoryItemContextMenu(KFileItemActions &fileItemActions)
 {
     // insert 'Open in new window' and 'Open in new tab' entries
-
     const KFileItemListProperties& selectedItemsProps = selectedItemsProperties();
-
-    addAction(m_mainWindow->actionCollection()->action(QStringLiteral("open_in_new_tab")));
-    addAction(m_mainWindow->actionCollection()->action(QStringLiteral("open_in_new_window")));
+    if (ContextMenuSettings::showOpenInNewTab()) {
+        addAction(m_mainWindow->actionCollection()->action(QStringLiteral("open_in_new_tab")));
+    }
+    if (ContextMenuSettings::showOpenInNewWindow()) {
+        addAction(m_mainWindow->actionCollection()->action(QStringLiteral("open_in_new_window")));
+    }
 
     // Insert 'Open With' entries
     addOpenWithActions(fileItemActions);
 
     // set up 'Create New' menu
-     DolphinNewFileMenu* newFileMenu = new DolphinNewFileMenu(m_mainWindow->actionCollection(), m_mainWindow);
-     const DolphinView* view = m_mainWindow->activeViewContainer()->view();
-     newFileMenu->setViewShowsHiddenFiles(view->hiddenFilesShown());
-     newFileMenu->checkUpToDate();
-     newFileMenu->setPopupFiles(QList<QUrl>() << m_fileInfo.url());
-     newFileMenu->setEnabled(selectedItemsProps.supportsWriting());
-     connect(newFileMenu, &DolphinNewFileMenu::fileCreated, newFileMenu, &DolphinNewFileMenu::deleteLater);
-     connect(newFileMenu, &DolphinNewFileMenu::directoryCreated, newFileMenu, &DolphinNewFileMenu::deleteLater);
+    DolphinNewFileMenu* newFileMenu = new DolphinNewFileMenu(m_mainWindow->actionCollection(), m_mainWindow);
+    const DolphinView* view = m_mainWindow->activeViewContainer()->view();
+    newFileMenu->setViewShowsHiddenFiles(view->hiddenFilesShown());
+    newFileMenu->checkUpToDate();
+    newFileMenu->setPopupFiles(QList<QUrl>() << m_fileInfo.url());
+    newFileMenu->setEnabled(selectedItemsProps.supportsWriting());
+    connect(newFileMenu, &DolphinNewFileMenu::fileCreated, newFileMenu, &DolphinNewFileMenu::deleteLater);
+    connect(newFileMenu, &DolphinNewFileMenu::directoryCreated, newFileMenu, &DolphinNewFileMenu::deleteLater);
 
-     QMenu* menu = newFileMenu->menu();
-     menu->setTitle(i18nc("@title:menu Create new folder, file, link, etc.", "Create New"));
-     menu->setIcon(QIcon::fromTheme(QStringLiteral("document-new")));
-     addMenu(menu);
+    QMenu* menu = newFileMenu->menu();
+    menu->setTitle(i18nc("@title:menu Create new folder, file, link, etc.", "Create New"));
+    menu->setIcon(QIcon::fromTheme(QStringLiteral("document-new")));
+    addMenu(menu);
 
-     addSeparator();
+    addSeparator();
 }
 
 void DolphinContextMenu::openItemContextMenu()
@@ -260,7 +273,7 @@ void DolphinContextMenu::openItemContextMenu()
             }
         }
 
-        if (selectionHasOnlyDirs) {
+        if (selectionHasOnlyDirs && ContextMenuSettings::showOpenInNewTab()) {
             // insert 'Open in new tab' entry
             addAction(m_mainWindow->actionCollection()->action(QStringLiteral("open_in_new_tabs")));
         }
@@ -270,24 +283,10 @@ void DolphinContextMenu::openItemContextMenu()
 
     insertDefaultItemActions(selectedItemsProps);
 
-    // insert 'Add to Places' entry if appropriate
-    if (m_selectedItems.count() == 1) {
-        if (m_fileInfo.isDir()) {
-            if (!placeExists(m_fileInfo.url())) {
-                addAction(m_mainWindow->actionCollection()->action(QStringLiteral("add_to_places")));
-            }
-        }
-    }
-
-    addSeparator();
-
-    fileItemActions.addServiceActionsTo(this);
-    fileItemActions.addPluginActionsTo(this);
-
-    addVersionControlPluginActions();
+    addAdditionalActions(fileItemActions, selectedItemsProps);
 
     // insert 'Copy To' and 'Move To' sub menus
-    if (GeneralSettings::showCopyMoveMenu()) {
+    if (ContextMenuSettings::showCopyMoveMenu()) {
         m_copyToMenu.setUrls(m_selectedItems.urlList());
         m_copyToMenu.setReadOnly(!selectedItemsProps.supportsWriting());
         m_copyToMenu.setAutoErrorHandlingEnabled(true);
@@ -337,23 +336,24 @@ void DolphinContextMenu::openViewportContextMenu()
     }
 
     // Insert 'Add to Places' entry if it's not already in the places panel
-    if (!placeExists(m_mainWindow->activeViewContainer()->url())) {
+    if (ContextMenuSettings::showAddToPlaces() &&
+            !placeExists(m_mainWindow->activeViewContainer()->url())) {
         addAction(m_mainWindow->actionCollection()->action(QStringLiteral("add_to_places")));
     }
     addSeparator();
 
     // Insert 'Sort By' and 'View Mode'
-    addAction(m_mainWindow->actionCollection()->action(QStringLiteral("sort")));
-    addAction(m_mainWindow->actionCollection()->action(QStringLiteral("view_mode")));
-
-    addSeparator();
-
-    // Insert service actions
-    fileItemActions.addServiceActionsTo(this);
-    fileItemActions.addPluginActionsTo(this);
-
-    addVersionControlPluginActions();
+    if (ContextMenuSettings::showSortBy()) {
+        addAction(m_mainWindow->actionCollection()->action(QStringLiteral("sort")));
+    }
+    if (ContextMenuSettings::showViewMode()) {
+        addAction(m_mainWindow->actionCollection()->action(QStringLiteral("view_mode")));
+    }
+    if (ContextMenuSettings::showSortBy() || ContextMenuSettings::showViewMode()) {
+        addSeparator();
+    }
 
+    addAdditionalActions(fileItemActions, baseUrlProperties);
     addCustomActions();
 
     addSeparator();
@@ -373,20 +373,34 @@ void DolphinContextMenu::insertDefaultItemActions(const KFileItemListProperties&
     // Insert 'Cut', 'Copy', 'Copy Location' and 'Paste'
     addAction(collection->action(KStandardAction::name(KStandardAction::Cut)));
     addAction(collection->action(KStandardAction::name(KStandardAction::Copy)));
-    QAction* copyPathAction = collection->action(QString("copy_location"));
-    copyPathAction->setEnabled(m_selectedItems.size() == 1);
-    addAction(copyPathAction);
+    if (ContextMenuSettings::showCopyLocation()) {
+        QAction* copyPathAction = collection->action(QString("copy_location"));
+        copyPathAction->setEnabled(m_selectedItems.size() == 1);
+        addAction(copyPathAction);
+    }
     QAction* pasteAction = createPasteAction();
     if (pasteAction) {
         addAction(pasteAction);
     }
-    addAction(m_mainWindow->actionCollection()->action(QStringLiteral("duplicate")));
 
-    addSeparator();
+    // Insert 'Duplicate Here'
+    if (ContextMenuSettings::showDuplicateHere()) {
+        addAction(m_mainWindow->actionCollection()->action(QStringLiteral("duplicate")));
+    }
 
     // Insert 'Rename'
     addAction(collection->action(KStandardAction::name(KStandardAction::RenameFile)));
 
+    // Insert 'Add to Places' entry if appropriate
+    if (ContextMenuSettings::showAddToPlaces() &&
+            m_selectedItems.count() == 1 &&
+            m_fileInfo.isDir() &&
+            !placeExists(m_fileInfo.url())) {
+        addAction(m_mainWindow->actionCollection()->action(QStringLiteral("add_to_places")));
+    }
+
+    addSeparator();
+
     // Insert 'Move to Trash' and/or 'Delete'
     const bool showDeleteAction = (KSharedConfig::openConfig()->group("KDE").readEntry("ShowDeleteCommand", false) ||
                                     !properties.isLocal());
@@ -484,8 +498,21 @@ void DolphinContextMenu::addOpenWithActions(KFileItemActions& fileItemActions)
     fileItemActions.addOpenWithActionsTo(this, QStringLiteral("DesktopEntryName != '%1'").arg(qApp->desktopFileName()));
 }
 
-void DolphinContextMenu::addVersionControlPluginActions()
+void DolphinContextMenu::addCustomActions()
 {
+    addActions(m_customActions);
+}
+
+void DolphinContextMenu::addAdditionalActions(KFileItemActions &fileItemActions, const KFileItemListProperties &props)
+{
+    addSeparator();
+
+    QList<QAction *> additionalActions;
+    if (props.isDirectory() && props.isLocal()) {
+        additionalActions << m_mainWindow->actionCollection()->action(QStringLiteral("open_terminal"));
+    }
+    fileItemActions.addActionsTo(this, KFileItemActions::MenuActionSource::All, additionalActions);
+
     const DolphinView* view = m_mainWindow->activeViewContainer()->view();
     const QList<QAction*> versionControlActions = view->versionControlActions(m_selectedItems);
     if (!versionControlActions.isEmpty()) {
@@ -494,8 +521,3 @@ void DolphinContextMenu::addVersionControlPluginActions()
     }
 }
 
-void DolphinContextMenu::addCustomActions()
-{
-    addActions(m_customActions);
-}
-
