@@ -39,10 +39,10 @@ public:
     void applySettings();
     void restoreDefaultSettings();
 
-signals:
+Q_SIGNALS:
     void changed();
 
-private slots:
+private Q_SLOTS:
 
     void slotDefaultSliderMoved(int value);
     void slotPreviewSliderMoved(int value);
@@ -64,6 +64,8 @@ private:
     QRadioButton* m_numberOfItems;
     QRadioButton* m_sizeOfContents;
     QSpinBox* m_recursiveDirectorySizeLimit;
+    QRadioButton* m_useRelatetiveDates;
+    QRadioButton* m_useShortDates;
 };
 
 #endif
