@@ -45,10 +45,13 @@
 #include <KProtocolManager>
 
 #include <QAbstractItemView>
+#include <QActionGroup>
 #include <QApplication>
 #include <QClipboard>
 #include <QDropEvent>
+#include <QGraphicsOpacityEffect>
 #include <QGraphicsSceneDragDropEvent>
+#include <QLabel>
 #include <QMenu>
 #include <QMimeDatabase>
 #include <QPixmapCache>
@@ -65,6 +68,7 @@ DolphinView::DolphinView(const QUrl& url, QWidget* parent) :
     m_assureVisibleCurrentIndex(false),
     m_isFolderWritable(true),
     m_dragging(false),
+    m_loading(false),
     m_url(url),
     m_viewPropertiesContext(),
     m_mode(DolphinView::IconsView),
@@ -82,7 +86,8 @@ DolphinView::DolphinView(const QUrl& url, QWidget* parent) :
     m_clearSelectionBeforeSelectingNewItems(false),
     m_markFirstNewlySelectedItemAsCurrent(false),
     m_versionControlObserver(nullptr),
-    m_twoClicksRenamingTimer(nullptr)
+    m_twoClicksRenamingTimer(nullptr),
+    m_placeholderLabel(nullptr)
 {
     m_topLayout = new QVBoxLayout(this);
     m_topLayout->setSpacing(0);
@@ -120,6 +125,28 @@ DolphinView::DolphinView(const QUrl& url, QWidget* parent) :
     connect(m_container->horizontalScrollBar(), &QScrollBar::valueChanged, this, [=] { hideToolTip(); });
     connect(m_container->verticalScrollBar(), &QScrollBar::valueChanged, this, [=] { hideToolTip(); });
 
+    // Show some placeholder text for empty folders
+    // This is made using a heavily-modified QLabel rather than a KTitleWidget
+    // because KTitleWidget can't be told to turn off mouse-selectable text
+    m_placeholderLabel = new QLabel(this);
+    QFont placeholderLabelFont;
+    // To match the size of a level 2 Heading/KTitleWidget
+    placeholderLabelFont.setPointSize(qRound(placeholderLabelFont.pointSize() * 1.3));
+    m_placeholderLabel->setFont(placeholderLabelFont);
+    m_placeholderLabel->setTextInteractionFlags(Qt::NoTextInteraction);
+    m_placeholderLabel->setWordWrap(true);
+    m_placeholderLabel->setAlignment(Qt::AlignCenter);
+    // Match opacity of QML placeholder label component
+    auto *effect = new QGraphicsOpacityEffect(m_placeholderLabel);
+    effect->setOpacity(0.5);
+    m_placeholderLabel->setGraphicsEffect(effect);
+    // Set initial text and visibility
+    updatePlaceholderLabel();
+
+    auto *centeringLayout = new QVBoxLayout(m_container);
+    centeringLayout->addWidget(m_placeholderLabel);
+    centeringLayout->setAlignment(m_placeholderLabel, Qt::AlignCenter);
+
     controller->setSelectionBehavior(KItemListController::MultiSelection);
     connect(controller, &KItemListController::itemActivated, this, &DolphinView::slotItemActivated);
     connect(controller, &KItemListController::itemsActivated, this, &DolphinView::slotItemsActivated);
@@ -140,7 +167,7 @@ DolphinView::DolphinView(const QUrl& url, QWidget* parent) :
 
     connect(m_model, &KFileItemModel::directoryLoadingStarted,       this, &DolphinView::slotDirectoryLoadingStarted);
     connect(m_model, &KFileItemModel::directoryLoadingCompleted,     this, &DolphinView::slotDirectoryLoadingCompleted);
-    connect(m_model, &KFileItemModel::directoryLoadingCanceled,      this, &DolphinView::directoryLoadingCanceled);
+    connect(m_model, &KFileItemModel::directoryLoadingCanceled,      this, &DolphinView::slotDirectoryLoadingCanceled);
     connect(m_model, &KFileItemModel::directoryLoadingProgress,   this, &DolphinView::directoryLoadingProgress);
     connect(m_model, &KFileItemModel::directorySortingProgress,   this, &DolphinView::directorySortingProgress);
     connect(m_model, &KFileItemModel::itemsChanged,
@@ -152,6 +179,9 @@ DolphinView::DolphinView(const QUrl& url, QWidget* parent) :
     connect(m_model, &KFileItemModel::directoryRedirection, this, &DolphinView::slotDirectoryRedirection);
     connect(m_model, &KFileItemModel::urlIsFileError,            this, &DolphinView::urlIsFileError);
 
+    connect(this, &DolphinView::itemCountChanged,
+            this, &DolphinView::updatePlaceholderLabel);
+
     m_view->installEventFilter(this);
     connect(m_view, &DolphinItemListView::sortOrderChanged,
             this, &DolphinView::slotSortOrderChangedByHeader);
@@ -499,17 +529,18 @@ QStringList DolphinView::mimeTypeFilters() const
     return m_model->mimeTypeFilters();
 }
 
-QString DolphinView::statusBarText() const
+void DolphinView::requestStatusBarText()
 {
-    QString summary;
-    QString foldersText;
-    QString filesText;
-
-    int folderCount = 0;
-    int fileCount = 0;
-    KIO::filesize_t totalFileSize = 0;
+    if (m_statJobForStatusBarText) {
+        // Kill the pending request.
+        m_statJobForStatusBarText->kill();
+    }
 
     if (m_container->controller()->selectionManager()->hasSelection()) {
+        int folderCount = 0;
+        int fileCount = 0;
+        KIO::filesize_t totalFileSize = 0;
+
         // Give a summary of the status of the selected files
         const KFileItemList list = selectedItems();
         for (const KFileItem& item : list) {
@@ -523,14 +554,37 @@ QString DolphinView::statusBarText() const
 
         if (folderCount + fileCount == 1) {
             // If only one item is selected, show info about it
-            return list.first().getStatusBarInfo();
+            Q_EMIT statusBarTextChanged(list.first().getStatusBarInfo());
         } else {
             // At least 2 items are selected
-            foldersText = i18ncp("@info:status", "1 Folder selected", "%1 Folders selected", folderCount);
-            filesText = i18ncp("@info:status", "1 File selected", "%1 Files selected", fileCount);
+            emitStatusBarText(folderCount, fileCount, totalFileSize, HasSelection);
         }
+    } else { // has no selection
+        if (!m_model->rootItem().url().isValid()) {
+            return;
+        }
+
+        m_statJobForStatusBarText = KIO::statDetails(m_model->rootItem().url(),
+                        KIO::StatJob::SourceSide, KIO::StatRecursiveSize, KIO::HideProgressInfo);
+        connect(m_statJobForStatusBarText, &KJob::result,
+                this, &DolphinView::slotStatJobResult);
+        m_statJobForStatusBarText->start();
+    }
+}
+
+void DolphinView::emitStatusBarText(const int folderCount, const int fileCount,
+                                    KIO::filesize_t totalFileSize, const Selection selection)
+{
+    QString foldersText;
+    QString filesText;
+    QString summary;
+
+    if (selection == HasSelection) {
+        // At least 2 items are selected because the case of 1 selected item is handled in
+        // DolphinView::requestStatusBarText().
+        foldersText = i18ncp("@info:status", "1 Folder selected", "%1 Folders selected", folderCount);
+        filesText = i18ncp("@info:status", "1 File selected", "%1 Files selected", fileCount);
     } else {
-        calculateItemCount(fileCount, folderCount, totalFileSize);
         foldersText = i18ncp("@info:status", "1 Folder", "%1 Folders", folderCount);
         filesText = i18ncp("@info:status", "1 File", "%1 Files", fileCount);
     }
@@ -548,8 +602,7 @@ QString DolphinView::statusBarText() const
     } else {
         summary = i18nc("@info:status", "0 Folders, 0 Files");
     }
-
-    return summary;
+    Q_EMIT statusBarTextChanged(summary);
 }
 
 QList<QAction*> DolphinView::versionControlActions(const KFileItemList& items) const
@@ -915,7 +968,7 @@ void DolphinView::slotItemsActivated(const KItemSet& indexes)
         const QUrl& url = openItemAsFolderUrl(item);
 
         if (!url.isEmpty()) { // Open folders in new tabs
-            Q_EMIT tabRequested(url, DolphinTabWidget::AfterLastTab);
+            Q_EMIT tabRequested(url);
         } else {
             items.append(item);
         }
@@ -933,9 +986,9 @@ void DolphinView::slotItemMiddleClicked(int index)
     const KFileItem& item = m_model->fileItem(index);
     const QUrl& url = openItemAsFolderUrl(item);
     if (!url.isEmpty()) {
-        Q_EMIT tabRequested(url, DolphinTabWidget::AfterCurrentTab);
+        Q_EMIT tabRequested(url);
     } else if (isTabsForFilesEnabled()) {
-        Q_EMIT tabRequested(item.url(), DolphinTabWidget::AfterCurrentTab);
+        Q_EMIT tabRequested(item.url());
     }
 }
 
@@ -1242,6 +1295,36 @@ void DolphinView::emitSelectionChangedSignal()
     Q_EMIT selectionChanged(selectedItems());
 }
 
+void DolphinView::slotStatJobResult(KJob *job)
+{
+    int folderCount = 0;
+    int fileCount = 0;
+    KIO::filesize_t totalFileSize = 0;
+    bool countFileSize = true;
+
+    const auto entry =  static_cast<KIO::StatJob *>(job)->statResult();
+    if (entry.contains(KIO::UDSEntry::UDS_RECURSIVE_SIZE)) {
+        // We have a precomputed value.
+        totalFileSize = static_cast<KIO::filesize_t>(
+                                entry.numberValue(KIO::UDSEntry::UDS_RECURSIVE_SIZE));
+        countFileSize = false;
+    }
+
+    const int itemCount = m_model->count();
+    for (int i = 0; i < itemCount; ++i) {
+        const KFileItem item = m_model->fileItem(i);
+        if (item.isDir()) {
+            ++folderCount;
+        } else {
+            ++fileCount;
+            if (countFileSize) {
+                totalFileSize += item.size();
+            }
+        }
+    }
+    emitStatusBarText(folderCount, fileCount, totalFileSize, NoSelection);
+}
+
 void DolphinView::updateSortRole(const QByteArray& role)
 {
     ViewProperties props(viewPropertiesUrl());
@@ -1508,40 +1591,6 @@ void DolphinView::hideToolTip(const ToolTipManager::HideBehavior behavior)
 #endif
 }
 
-void DolphinView::calculateItemCount(int& fileCount,
-                                     int& folderCount,
-                                     KIO::filesize_t& totalFileSize) const
-{
-    const int itemCount = m_model->count();
-
-    bool countFileSize = true;
-
-    if (!m_model->rootItem().url().isValid()) {
-        return;
-    }
-
-    // In case we have a precomputed value
-    const auto job = KIO::statDetails(m_model->rootItem().url(), KIO::StatJob::SourceSide, KIO::StatRecursiveSize, KIO::HideProgressInfo);
-    job->exec();
-    const auto entry =  job->statResult();
-    if (entry.contains(KIO::UDSEntry::UDS_RECURSIVE_SIZE)) {
-        totalFileSize = static_cast<KIO::filesize_t>(entry.numberValue(KIO::UDSEntry::UDS_RECURSIVE_SIZE));
-        countFileSize = false;
-    }
-
-    for (int i = 0; i < itemCount; ++i) {
-        const KFileItem item = m_model->fileItem(i);
-        if (item.isDir()) {
-            ++folderCount;
-        } else {
-            ++fileCount;
-            if (countFileSize) {
-                totalFileSize += item.size();
-            }
-        }
-    }
-}
-
 void DolphinView::slotTwoClicksRenamingTimerTimeout()
 {
     const KItemListSelectionManager* selectionManager = m_container->controller()->selectionManager();
@@ -1594,6 +1643,9 @@ void DolphinView::slotRenamingResult(KJob* job)
 
 void DolphinView::slotDirectoryLoadingStarted()
 {
+    m_loading = true;
+    updatePlaceholderLabel();
+
     // Disable the writestate temporary until it can be determined in a fast way
     // in DolphinView::slotDirectoryLoadingCompleted()
     if (m_isFolderWritable) {
@@ -1606,15 +1658,30 @@ void DolphinView::slotDirectoryLoadingStarted()
 
 void DolphinView::slotDirectoryLoadingCompleted()
 {
+    m_loading = false;
+
     // Update the view-state. This has to be done asynchronously
     // because the view might not be in its final state yet.
     QTimer::singleShot(0, this, &DolphinView::updateViewState);
 
+    // Update the placeholder label in case we found that the folder was empty
+    // after loading it
+
     Q_EMIT directoryLoadingCompleted();
 
+    updatePlaceholderLabel();
     updateWritableState();
 }
 
+void DolphinView::slotDirectoryLoadingCanceled()
+{
+    m_loading = false;
+
+    updatePlaceholderLabel();
+
+    Q_EMIT directoryLoadingCanceled();
+}
+
 void DolphinView::slotItemsChanged()
 {
     m_assureVisibleCurrentIndex = false;
@@ -1976,3 +2043,35 @@ void DolphinView::slotSwipeUp()
 {
     Q_EMIT goUpRequested();
 }
+
+void DolphinView::updatePlaceholderLabel()
+{
+    if (m_loading || itemsCount() > 0) {
+        m_placeholderLabel->setVisible(false);
+        return;
+    }
+
+    if (!nameFilter().isEmpty()) {
+        m_placeholderLabel->setText(i18n("No items matching the filter"));
+    } else if (m_url.scheme() == QLatin1String("baloosearch") || m_url.scheme() == QLatin1String("filenamesearch")) {
+        m_placeholderLabel->setText(i18n("No items matching the search"));
+    } else if (m_url.scheme() == QLatin1String("trash")) {
+        m_placeholderLabel->setText(i18n("Trash is empty"));
+    } else if (m_url.scheme() == QLatin1String("tags")) {
+        m_placeholderLabel->setText(i18n("No tags"));
+    } else if (m_url.scheme() == QLatin1String("recentlyused")) {
+        m_placeholderLabel->setText(i18n("No recently used items"));
+    } else if (m_url.scheme() == QLatin1String("smb")) {
+        m_placeholderLabel->setText(i18n("No shared folders found"));
+    } else if (m_url.scheme() == QLatin1String("network")) {
+        m_placeholderLabel->setText(i18n("No relevant network resources found"));
+    } else if (m_url.scheme() == QLatin1String("mtp")) {
+        m_placeholderLabel->setText(i18n("No MTP-compatible devices found"));
+    } else if (m_url.scheme() == QLatin1String("bluetooth")) {
+        m_placeholderLabel->setText(i18n("No Bluetooth devices found"));
+    } else {
+        m_placeholderLabel->setText(i18n("Folder is empty"));
+    }
+
+    m_placeholderLabel->setVisible(true);
+}
